#ifndef CPP_DATASTRUCTURES_GENARRAY_H
#define CPP_DATASTRUCTURES_GENARRAY_H

// Eine (sehr) generische Array Klasse fr dynamische Arrays und
// ein daraus abgeleitetes template array.
//
// In diesem Array drfen nur Elemente liegen, die mit memcpy()
// kopiert werden knnen (d.h. keine Objekte mit einem eigenen
// Kopierkonstruktor).
//
// Die Vergrerungsstrategie ist simpel:
//   Entweder wird die Arraygre verdoppelt, oder, falls der
//   bentigte Index grer als die alte doppelte Gre ist,
//   das Array gerade so gro gemacht, da der Index im Array
//   liegt.
//
// Fehlender Speicher wird komplett ber Ausnahmen (MemoryX) behandelt.
//
// Autor: Jochen Becher
//
// Historie:
// Version 1.0, am 25. Juni 94
//

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif

#ifndef CPP_EXCEPTIONS_EXCEPTIONS_H
#include <classes/exceptions/exceptions.h>
#endif

class gen_array {
public:
	// 'es' bezeichnet die Gre der Eintrge (sizeof())
 	// 's' gibt eine Vorgabe fr die Gre des Arrays
	gen_array(ULONG es, ULONG s = 16) throw (MemoryX);
	gen_array(const gen_array &) throw (MemoryX);
	gen_array &operator= (const gen_array &) throw (MemoryX);
	~gen_array();
	operator APTR () const { return (APTR) v; };
	ULONG esize() const { return element_size; };
	ULONG count() const { return size; };
	// die Referenz mu auf den passenden Typ gecastet werden
	VOID & operator[] (ULONG i) throw (MemoryX);
	// die Anzahl der Feldelemente, der grte Index auf den ohne
	// Feldvergrerung zugegriffen werden kann ist somit count-1
private:
	VOID extend(ULONG) throw (MemoryX);
	UBYTE *v;
	ULONG element_size;
	ULONG size;
};

template <class T> class array : private gen_array {
public:
	array(ULONG s = 16) throw (MemoryX);
	ULONG count() const { return gen_array::count(); };
	T &operator[] (ULONG i)  throw (MemoryX);
};

// *************************************************************

template <class T> array<T>::array(ULONG s)
	: gen_array(sizeof(T),s)
{
}

template <class T> T &array<T>::operator[] (ULONG i)
{
	return (T &) gen_array::operator[] (i);
}

#endif
